@@ -1,18 +1,7 @@
 # -*- coding: utf-8 -*-
 __author__ = "Javier de la Rosa"
-__credits__ = [
-    "Javier de la Rosa",
-    "Diego Muñoz Escalante, https://github.com/escalant3",
-    "Yashh, https://github.com/yashh",
-    "timClicks, https://github.com/timClicks",
-    "Mark Henderson, https://github.com/emehrkay",
-    "Johan Lundberg, https://github.com/johanlundberg",
-    "Mark Ng, https://github.com/markng",
-    "kesavkolla, https://github.com/kesavkolla",
-    "Andy Denmark, https://github.com/denmark",
-]
 __license__ = "GPL 3"
-__version__ = "1.5.0"
+__version__ = "1.6.0"
 __email__ = "versae@gmail.com"
 __url__ = "https://github.com/versae/neo4j-rest-client"
 __description__ = """Object-oriented Python library to interact with """ \
@@ -45,7 +34,12 @@ RELATIONSHIP = "relationship"
 PATH = "path"
 FULLPATH = "fullpath"
 POSITION = "position"
+RAW = "raw"
+ITERABLE = "iterable"
 # Indexes
+INDEX = "index"
+INDEX_NODE = "index_node"
+INDEX_RELATIONSHIP = "index_relationship"
 INDEX_EXACT = "exact"
 INDEX_FULLTEXT = "fulltext"
 # Transactions
