@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from datetime import datetime
 try:
     import cPickle as pickle
 except:
@@ -9,8 +10,6 @@ import constants
 import request
 import unittest
 
-from lucenequerybuilder import Q
-
 
 class NodesTestCase(unittest.TestCase):
 
@@ -18,6 +17,10 @@ class NodesTestCase(unittest.TestCase):
         self.url = "http://localhost:7474/db/data/"
         self.gdb = client.GraphDatabase(self.url)
 
+    def tearDown(self):
+        if self.gdb:
+            self.gdb.flush()
+
     def test_connection_cache(self):
         import options as clientCache
         clientCache.CACHE = True
@@ -244,6 +247,13 @@ class IndexesTestCase(RelationshipsTestCase):
         index["surnames"]["d"] = n1
         self.assertTrue(n1 in index["surnames"]["d"])
 
+    def test_create_index_for_nodes_and_dots(self):
+        # From https://github.com/versae/neo4j-rest-client/issues/43
+        n1 = self.gdb.nodes.create(name="John.Doe", place="Texas.s")
+        index = self.gdb.nodes.indexes.create(name="doe")
+        index["surnames.s"]["d.d"] = n1
+        self.assertTrue(n1 in index["surnames.s"]["d.d"])
+
     def test_create_index_for_nodes_unicode(self):
         n1 = self.gdb.nodes.create(name="Lemmy", band="Motörhead")
         index = self.gdb.nodes.indexes.create(name="doe")
@@ -306,6 +316,7 @@ class IndexesTestCase(RelationshipsTestCase):
                           index["feeling"].__getitem__, "hate")
 
     def test_query_index(self):
+        Q = client.Q
         n1 = self.gdb.nodes.create(name="John Doe", place="Texas")
         n2 = self.gdb.nodes.create(name="Michael Donald", place="Tijuana")
         index = self.gdb.nodes.indexes.create(name="do", type="fulltext")
@@ -340,6 +351,54 @@ class TraversalsTestCase(IndexesTestCase):
         traversal = n1.traverse(types=types)
         self.assertTrue(len(traversal) > 0)
 
+    def test_path_traversal(self):
+        """
+        Tests the use of Path as returnable type.
+        """
+        nodes = [self.gdb.nodes.create() for i in xrange(10)]
+        # Chain them into a linked list
+        last = None
+        for n in nodes:
+            if last:
+                last.relationships.create("Knows", n)
+            last = n
+        # Toss in a different relationship type to ensure the
+        # STOP_AT_END_OF_GRAPH didn't break traversing by type
+        nodes[-1].relationships.create("Test", self.gdb.nodes.create())
+        types = [
+            client.All.Knows,
+        ]
+        stop = constants.STOP_AT_END_OF_GRAPH
+        traversal = nodes[0].traverse(types=types, stop=stop,
+                                      returns=constants.PATH)
+        paths = [path for path in traversal]
+        self.assertTrue(len(traversal) > 0)
+        self.assertTrue(all([isinstance(path, client.Path) for path in paths]))
+        start_node = paths[0].start
+        relationship = paths[0].relationships[0]
+        self.assertTrue(isinstance(start_node, client.Node))
+        self.assertTrue(isinstance(relationship, client.Relationship))
+
+    def test_path_traversal_getitem(self):
+        # Test from @shahin: https://gist.github.com/1418704
+        n1 = self.gdb.node()
+        n2 = self.gdb.node()
+        rel = self.gdb.relationships.create(n1,'knows',n2)
+        # Define path traverser
+        class PathTraverser(self.gdb.Traversal):
+            stop = constants.STOP_AT_END_OF_GRAPH
+            returns = constants.PATH
+        try:
+            paths = [traversal for traversal in PathTraverser(n1)]
+            self.assertEqual(len(paths[0]), 1)
+        except:
+            raise
+        finally:
+            # Clean up on fail
+            self.gdb.relationships.delete(rel.id)
+            self.gdb.node.delete(n2.id)
+            self.gdb.node.delete(n1.id)
+
     def test_graph_wide_traversal(self):
         """
         Tests the use of constants.STOP_AT_END_OF_GRAPH as a stop depth.
@@ -405,6 +464,8 @@ class TraversalsTestCase(IndexesTestCase):
         """
         Tests the use of paginated traversals.
         """
+        import options as clientDebug
+        clientDebug.DEBUG = False
         nodes = [self.gdb.nodes.create() for i in xrange(10)]
         # Chain them into a linked list
         last = None
@@ -438,6 +499,129 @@ class TraversalsTestCase(IndexesTestCase):
         traversal_length = len([n for n in [t for t in pages]])
         self.assertEqual(traversal_length, 0)
 
+    # Taken from the official tests by Neo4j python-embedded
+    # https://github.com/neo4j/python-embedded/blob/master/src/test/python/traversal.py
+    def create_data(self):
+        with self.gdb.transaction():
+            self.source = self.gdb.node(message='hello')
+            target = self.gdb.node(message='world')
+            relationship = self.source.related_to(target, message="graphy")
+            secondrel = target.likes(self.source, message="buh")
+
+    def test_traverse_string_types(self):
+        self.create_data()
+        db = self.gdb
+        start_node = self.source
+        # START SNIPPET: basicTraversal
+        traverser = db.traversal()\
+            .relationships('related_to')\
+            .traverse(start_node)
+        # The graph is traversed as 
+        # you loop through the result.
+        for node in traverser.nodes:
+            pass
+        # END SNIPPET: basicTraversal
+        self.assertEqual(len(list(traverser.nodes)), 1)
+        # START SNIPPET: directedTraversal
+        from traversals import RelationshipDirection
+        OUTGOING = RelationshipDirection.OUTGOING
+        INCOMING = RelationshipDirection.INCOMING
+        ANY = RelationshipDirection.ANY
+        traverser = db.traversal()\
+            .relationships('related_to', OUTGOING)\
+            .traverse(start_node)
+        # END SNIPPET: directedTraversal
+        self.assertEqual(len(list(traverser.nodes)), 1)
+        traverser = db.traversal()\
+            .relationships('related_to', INCOMING)\
+            .relationships('likes')\
+            .traverse(start_node)
+        # END SNIPPET: multiRelationshipTraversal
+        self.assertEqual(len(list(traverser.nodes)), 1)
+        # START SNIPPET: traversalResults
+        traverser = db.traversal()\
+            .relationships('related_to')\
+            .traverse(start_node)
+        # Get each possible path
+        for path in traverser:
+            pass
+        # Get each node
+        for node in traverser.nodes:
+            pass
+        # Get each relationship
+        for relationship in traverser.relationships:
+            pass
+        # END SNIPPET: traversalResults
+
+    def test_traverse_programmatic_types(self):
+        from client import Direction
+        self.create_data()
+        t = self.gdb.traversal()\
+            .depthFirst()\
+            .relationships(Direction.ANY.related_to)\
+            .traverse(self.source)
+        res = list(t.nodes)
+        self.assertEqual(len(res), 1)
+
+    def test_uniqueness(self):
+        self.create_data()
+        db = self.gdb
+        start_node = self.source
+        # START SNIPPET: uniqueness
+        from traversals import Uniqueness
+        traverser = db.traversal()\
+            .uniqueness(Uniqueness.NODE_PATH)\
+            .traverse(start_node)
+        # END SNIPPET: uniqueness
+        res = list(traverser.nodes)
+        self.assertEqual(len(res), 2)
+
+    def test_ordering(self):
+        self.create_data()
+        db = self.gdb
+        start_node = self.source
+        # START SNIPPET: ordering
+        # Depth first traversal, this
+        # is the default.
+        traverser = db.traversal()\
+            .depthFirst()\
+            .traverse(self.source)
+        # Breadth first traversal
+        traverser = db.traversal()\
+            .breadthFirst()\
+            .traverse(start_node)
+        # END SNIPPET: ordering
+        res = list(traverser.nodes)
+        self.assertEqual(len(res), 1)
+
+    def test_paths(self):
+        self.create_data()
+        t = self.gdb.traversal()\
+            .traverse(self.source)
+        for path in t:
+            # START SNIPPET: accessPathStartAndEndNode
+            start_node = path.start
+            end_node = path.end
+            # END SNIPPET: accessPathStartAndEndNode
+            self.assertNotEqual(start_node, None)
+            self.assertNotEqual(end_node, None)
+            # START SNIPPET: accessPathLastRelationship
+            last_relationship = path.last_relationship
+            # END SNIPPET: accessPathLastRelationship
+            # START SNIPPET: loopThroughPath
+            for item in path:
+                # Item is either a Relationship,
+                # or a Node
+                pass
+            for nodes in path.nodes:
+                # All nodes in a path
+                pass
+            for nodes in path.relationships:
+                # All relationships in a path
+                pass
+            # END SNIPPET: loopThroughPath
+            break
+
 
 class ExtensionsTestCase(TraversalsTestCase):
 
@@ -491,11 +675,20 @@ class ExtensionsTestCase(TraversalsTestCase):
         n1.relationships.create("related", n2)
         n1.relationships.create("related", n3)
         gremlin = self.gdb.extensions.GremlinPlugin.execute_script
+        import options as clientDebug
+        clientDebug.DEBUG = True
         rels = gremlin(script='g.v(%s).outE' % n1.id,
                        returns=constants.RELATIONSHIP)
         self.assertEqual(len(rels), 2)
         for rel in rels:
             self.assertTrue(isinstance(rel, client.Relationship))
+        clientDebug.DEBUG = False
+
+    def test_gremlin_extension_reference_raw_returns(self):
+        # Assuming the GremlinPlugin installed
+        ext = self.gdb.extensions.GremlinPlugin
+        n = ext.execute_script(script='g.v(0)', returns=constants.RAW)
+        self.assertTrue(isinstance(n, dict))
 
 
 class TransactionsTestCase(ExtensionsTestCase):
@@ -548,6 +741,25 @@ class TransactionsTestCase(ExtensionsTestCase):
         self.assertTrue(isinstance(n, client.Node))
         self.assertTrue(n.get("age", True))
 
+    def test_transaction_create_and_set(self):
+        with self.gdb.transaction():
+            n = self.gdb.nodes.create(age=25)
+            n.set("surname", u"AC/DC")
+            n["name"] = u"Motörhead"
+            self.assertEqual(n.properties, {
+                "age": 25,
+                "name": u"Motörhead",
+                "surname": u"AC/DC",
+            })
+        self.assertTrue(isinstance(n, client.Node))
+        self.assertEqual(n.get("age"), 25)
+        self.assertEqual(n.get("name"), u"Motörhead")
+        self.assertEqual(n.properties, {
+            "age": 25,
+            "name": u"Motörhead",
+            "surname": u"AC/DC",
+        })
+
     def test_transaction_get(self):
         n1 = self.gdb.nodes.get(1)
         with self.gdb.transaction():
@@ -666,6 +878,226 @@ class TransactionsTestCase(ExtensionsTestCase):
             self.assertTrue(isinstance(node, client.Node))
             self.assertEqual(position, node["position"])
 
+    def test_transaction_conections(self):
+        import options as clientDebug
+        clientDebug.DEBUG = True
+        nodes = []
+        with self.gdb.transaction(commit=False) as tx:
+            for i in range(5):
+                nodes.append(self.gdb.node[i])
+        tx.commit()
+        clientDebug.DEBUG = False
+
+    def test_transaction_create_relationship_functional(self):
+        with self.gdb.transaction():
+            n1 = self.gdb.node()
+            n2 = self.gdb.node()
+            rel = n1.relationships.create("Knows", n2)
+            rel["when"] = "January"
+        self.assertEqual(rel.properties, {"when": "January"})
+
+    def test_transaction_create_relationship_functional_mixed2(self):
+        n1 = self.gdb.node()
+        with self.gdb.transaction():
+            n2 = self.gdb.node()
+            rel = n1.relationships.create("Knows", n2)
+            rel["when"] = "January"
+        self.assertEqual(rel.properties, {"when": "January"})
+
+    def test_transaction_create_relationship_functional_mixed2(self):
+        n2 = self.gdb.node()
+        with self.gdb.transaction():
+            n1 = self.gdb.node()
+            rel = n1.relationships.create("Knows", n2)
+            rel["when"] = "January"
+        self.assertEqual(rel.properties, {"when": "January"})
+
+    def test_transaction_create_relationship_functional(self):
+        with self.gdb.transaction():
+            n1 = self.gdb.node()
+            n2 = self.gdb.node()
+            rel = n1.relationships.create("Knows", n2)
+            rel["when"] = "January"
+            self.assertEqual(rel.start, n1)
+            self.assertEqual(rel.end, n2)
+        self.assertEqual(rel.properties, {"when": "January"})
+        self.assertTrue(isinstance(n1, client.Node))
+        self.assertTrue(isinstance(n2, client.Node))
+
+    def test_transaction_create_relationship(self):
+        with self.gdb.transaction():
+            n1 = self.gdb.node()
+            n2 = self.gdb.node()
+            rel = n1.Knows(n2)
+            rel["when"] = "January"
+        self.assertEqual(rel.properties, {"when": "January"})
+
+    # The next tests for transaction were taken from @mhluongo fork
+    # https://github.com/mhluongo/neo4j-rest-client/blob/master/neo4jrestclient/tests.py
+    def test_transaction_index_creation(self):
+        """
+        Tests whether indexes are properly created during a transaction. 
+        Asserts the creation also behaves transactionally (ie, not until
+        commit).
+        """
+        with self.gdb.transaction():
+            i1 = self.gdb.nodes.indexes.create('index_from_tx')
+            transactionality_test = i1()
+        self.assertTrue(isinstance(transactionality_test, dict))
+        self.assertTrue(i1 is not None)
+        self.assertTrue(isinstance(i1, client.Index))
+        i2 = self.gdb.nodes.indexes.get('index_from_tx')
+        self.assertTrue(i1 == i2)
+
+    def test_transaction_add_node_to_index(self):
+        """
+        Tests whether a node can be added to an index within a transaction.
+        Does not assert transactionality.
+        """
+        n1 = self.gdb.nodes.create()
+        index = self.gdb.nodes.indexes.create('index_nodes')
+        with self.gdb.transaction():
+            index.add('test1','test1', n1)
+            index['test2']['test2'] = n1
+        self.assertTrue(index['test1']['test1'][-1] == n1)
+        self.assertTrue(index['test2']['test2'][-1] == n1)
+
+    def test_transaction_index_add_rel_to_index(self):
+        """
+        Tests whether a relationship can be added to an index within a transaction.
+        Does not assert transactionality.
+        """
+        #test nodes
+        n1 = self.gdb.nodes.create()
+        n2 = self.gdb.nodes.create()
+        r = n1.relationships.create('Knows', n2)
+        index = self.gdb.relationships.indexes.create('index_rel')
+        with self.gdb.transaction():
+            index.add('test1','test1', r)
+            index['test2']['test2'] = r
+        self.assertTrue(index['test1']['test1'][-1] == r)
+        self.assertTrue(index['test2']['test2'][-1] == r)
+
+    def test_transaction_index_query(self):
+        """
+        Tests whether the transaction methods work with index queries.
+        Note- this test does not prove query transactionality.
+        """
+        n1 = self.gdb.nodes.create()
+        index = self.gdb.nodes.indexes.create('index2')
+        index.add('test2','test2', n1)
+        # Test getting nodes from index during transaction
+        tx = self.gdb.transaction()
+        index_hits = index['test2']['test2']
+        tx.commit()
+        self.assertTrue(n1 == index_hits[:][-1])
+
+    def test_transaction_remove_node_from_index(self):
+        index = self.gdb.nodes.indexes.create('index3')
+        n = self.gdb.nodes.create()
+        index.add('test3','test3', n)
+        tx = self.gdb.transaction(using_globals=False)
+        index.delete('test3', 'test3', n, tx=tx)
+        # Assert transactional
+        self.assertTrue(n in index['test3']['test3'])
+        tx.commit()
+        self.assertTrue(n not in index['test3']['test3'])
+
+    def test_transaction_query_index_for_new_node(self):
+        #test nodes created in transaction
+        
+        index = self.gdb.nodes.indexes.create('index4%s' \
+                                              % datetime.now().microsecond)
+        tx = self.gdb.transaction(using_globals=False)
+        n4 = self.gdb.nodes.create(tx=tx)
+        index.add('test3','test3', n4, tx=tx)
+        # Assert transactional
+        transactional = True
+        try:
+            index['test3']['test3'][0]
+            transactional = False
+        except:
+            pass
+        tx.commit()
+        self.assertTrue(transactional)
+        self.assertTrue(index['test3']['test3'][-1] == n4)
+
+    def test_transaction_add_to_new_index(self):
+        """
+        Tests whether a node can be added to an index that was created earlier
+        in the transaction.
+        Does not assert transactionality.
+        """
+        n1 = self.gdb.nodes.create()
+        tx = self.gdb.transaction()
+        index = self.gdb.nodes.indexes.create('index5')
+        index.add('test1','test1', n1)
+        tx.commit()
+        self.assertTrue(isinstance(index, client.Index))
+        self.assertTrue(index['test1']['test1'][-1] == n1)
+
+    def test_transaction_new_node_properties(self):
+        """
+        Tests setting properties on a node created within the same tx.
+        Doesn't show transactionality.
+        """
+        def has_props(node):
+            return n['name'] == 'test' and n['age'] == 0
+
+        tx = self.gdb.transaction()
+        n = self.gdb.node()
+        n['name'] = 'test'
+        n['age'] = 0
+        tx_props_kept = has_props(n)
+        tx.commit()
+        self.assertTrue(tx_props_kept)
+        self.assertTrue(has_props(n))
+
+    def test_transaction_properties_class(self):
+        def has_props(node):
+            return node['test1'] == 'test1' and \
+                   node['test2'] == 'test2'
+
+        def set_props(node):
+            node['test1'] = 'test1'
+            node['test2'] = 'test2'
+
+        n1 = self.gdb.node()
+        tx = self.gdb.transaction()
+        set_props(n1)
+        has_props_before_commit = has_props(n1)
+        tx.commit()
+        self.assertFalse(has_props_before_commit)
+        self.assertTrue(has_props(n1))
+        tx = self.gdb.transaction()
+        n2 = self.gdb.node()
+        set_props(n2)
+        has_props_before_commit = has_props(n1)
+        tx.commit()
+        self.assertFalse(has_props_before_commit)
+        self.assertTrue(has_props(n2))
+
+#    def test_transaction_traversal(self):
+#        nodes = [self.gdb.nodes.create() for i in xrange(10)]
+#        # Chain them into a linked list
+#        last = None
+#        for n in nodes:
+#            if last:
+#                last.relationships.create("Knows", n)
+#            last = n
+
+#        with self.gdb.transaction():
+#            tx_n = self.gdb.node()
+#            last.relationships.create('Knows', tx_n)
+#            types = [
+#                client.All.Knows,
+#            ]
+#            stop = constants.STOP_AT_END_OF_GRAPH
+#            traversal = nodes[0].traverse(types=types, stop=stop)
+
+#        #a non-transactional traversal will be 1 node short
+#        self.assertEqual(len(traversal), len(nodes))
+
 
 class PickleTestCase(TransactionsTestCase):
 
